using System;
using System.ComponentModel;
using System.Net;

using Smdn.Net.Imap4.Client;
using Smdn.Net.Pop3.Client;

namespace Smdn.Applications.MRAConnector {
  public class ConnectBackgroundWorker : BackgroundWorker {
    public ConnectBackgroundWorker()
    {
      WorkerReportsProgress = true;
      WorkerSupportsCancellation = true;
    }

    protected override void OnDoWork(DoWorkEventArgs e)
    {
      base.OnDoWork(e);

      var imapArgs = e.Argument as Tuple<ImapClientProfile, NetworkCredential, string[]>;
      var popArgs = e.Argument as Tuple<PopClientProfile, NetworkCredential, string[]>;

      ReportProgress(0);

      if (imapArgs != null) {
        using (var client = new ImapClient(imapArgs.Item1)) {
          client.Profile.UsingSaslMechanisms = imapArgs.Item3;

          ReportProgress(10);

          if (CancellationPending) {
            e.Cancel = true;
            return;
          }

          var asyncResult = client.BeginConnect(imapArgs.Item2);

          for (;;) {
            if (asyncResult.AsyncWaitHandle.WaitOne(250, false))
              break;

            if (CancellationPending) {
              e.Cancel = true;
              return;
            }
          }

          client.EndConnect(asyncResult);

          ReportProgress(90);

          if (CancellationPending) {
            e.Cancel = true;
            return;
          }

          client.Logout();
        }
      }
      else {
        using (var client = new PopClient(popArgs.Item1)) {
          client.Profile.UsingSaslMechanisms = popArgs.Item3;

          ReportProgress(10);

          if (CancellationPending) {
            e.Cancel = true;
            return;
          }

          var asyncResult = client.BeginConnect(popArgs.Item2);

          for (;;) {
            if (asyncResult.AsyncWaitHandle.WaitOne(250, false))
              break;

            if (CancellationPending) {
              e.Cancel = true;
              return;
            }
          }

          client.EndConnect(asyncResult);

          ReportProgress(90);

          if (CancellationPending) {
            e.Cancel = true;
            return;
          }

          client.Logout();
        }
      }

      ReportProgress(100);
    }

    protected override void OnProgressChanged(ProgressChangedEventArgs e)
    {
      base.OnProgressChanged(e);
    }

    protected override void OnRunWorkerCompleted(RunWorkerCompletedEventArgs e)
    {
      base.OnRunWorkerCompleted(e);
    }
  }
}
