using System;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;

using Smdn.Net;
using Smdn.Net.MessageAccessProtocols.Diagnostics;

namespace Smdn.Applications.MRAConnector {
  public class TextBoxTraceListener : TraceListener {
    public TextBoxTraceListener(string name, TextBox outputTextBox)
      : base(name)
    {
      this.outputTextBox = outputTextBox;

      Clear();
    }

    public void Clear()
    {
      builder.Length = 0;
      outputTextBox.Clear();
    }

    public override void TraceData(TraceEventCache eventCache, string source, TraceEventType eventType, int id, object data)
    {
      if (data is ReceiveTraceData) {
        var received = data as ReceiveTraceData;
        var recv = NetworkTransferEncoding.Transfer8Bit.GetString(received.Data.Array, received.Data.Offset, received.Data.Count);

        Write("S:\t");
        WriteLine(recv.TrimEnd().Replace("\r\n", "\r\n\t"));
        Flush();
      }
      else if (data is SendTraceData) {
        var sent = data as SendTraceData;
        var snt = NetworkTransferEncoding.Transfer8Bit.GetString(sent.Data.Array, sent.Data.Offset, sent.Data.Count);

        Write("C:\t");
        WriteLine(snt.TrimEnd().Replace("\r\n", "\r\n\t"));
        Flush();
      }
      /*
      else if (data is MessageTraceData) {
        WriteLine((data as MessageTraceData).Message);
      }
      else if (data is ExceptionTraceData) {
        WriteLine((data as ExceptionTraceData).Exception);
      }
      */
      else {
        // ignore
        //base.TraceData(eventCache, source, eventType, id, data);
      }
    }

    public override void Write(string message)
    {
      builder.Append(message);
    }

    public override void WriteLine(string message)
    {
      builder.AppendLine(message);
    }

    public override void Flush()
    {
      if (outputTextBox.InvokeRequired)
        outputTextBox.Invoke((MethodInvoker)UpdateText);
      else
        UpdateText();
    }

    private void UpdateText()
    {
      outputTextBox.Text = builder.ToString();
      outputTextBox.Refresh();
      outputTextBox.Select(builder.Length, 0);
      outputTextBox.ScrollToCaret();
    }

    private StringBuilder builder = new StringBuilder(1024 * 10);
    private TextBox outputTextBox;
  }
}
